"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const React = __importStar(require("react"));
const model_1 = require("./model");
const Diff_1 = require("./Diff");
const apputils_1 = require("@jupyterlab/apputils");
const coreutils_1 = require("@jupyterlab/coreutils");
const typestyle_1 = require("typestyle");
const git_1 = require("../../git");
/**
 * Method to open a main menu panel to show the diff of a given Notebook file.
 * If one already exists, just activates the existing one.
 *
 * @param filePath the path relative to the Git repo root.
 * @param app The JupyterLab application instance
 * @param diffContext the context in which the diff is being requested
 * @param renderMime the renderMime registry instance from the
 * @param topRepoPath the Git repo root path.
 */
function openDiffView(filePath, app, diffContext, renderMime, topRepoPath) {
    return __awaiter(this, void 0, void 0, function* () {
        if (Diff_1.isDiffSupported(filePath)) {
            const id = `nbdiff-${filePath}-${model_1.getRefValue(diffContext.currentRef)}`;
            let mainAreaItems = app.shell.widgets('main');
            let mainAreaItem = mainAreaItems.next();
            while (mainAreaItem) {
                if (mainAreaItem.id === id) {
                    app.shell.activateById(id);
                    break;
                }
                mainAreaItem = mainAreaItems.next();
            }
            if (!mainAreaItem) {
                const relativeFilePath = yield getRelativeFilePath(filePath, topRepoPath);
                const nbDiffWidget = apputils_1.ReactWidget.create(React.createElement(Diff_1.RenderMimeProvider, { value: renderMime },
                    React.createElement(Diff_1.Diff, { path: relativeFilePath, diffContext: diffContext })));
                nbDiffWidget.id = id;
                nbDiffWidget.title.label = coreutils_1.PathExt.basename(filePath);
                nbDiffWidget.title.iconClass = typestyle_1.style({
                    backgroundImage: 'var(--jp-icon-diff)'
                });
                nbDiffWidget.title.closable = true;
                nbDiffWidget.addClass('jp-git-diff-parent-diff-widget');
                app.shell.add(nbDiffWidget, 'main');
                app.shell.activateById(nbDiffWidget.id);
            }
        }
        else {
            apputils_1.showDialog({
                title: 'Diff Not Supported',
                body: `Diff is not supported for ${coreutils_1.PathExt.extname(filePath).toLocaleLowerCase()} files.`
            });
        }
    });
}
exports.openDiffView = openDiffView;
let serverRootResultCache = null;
/**
 * Gets the path of the file relative to the Jupyter server root. This resolves the server root path after calling
 * the `/git/server_root` REST API, and uses the Git repo root and the file path relative to the repo root to resolve
 * the rest.
 * @param path the file path relative to Git repo root
 * @param topRepoPath the Git repo root path
 */
function getRelativeFilePath(path, topRepoPath) {
    return __awaiter(this, void 0, void 0, function* () {
        if (serverRootResultCache !== null) {
            return serverRootResultCache;
        }
        else {
            const response = yield git_1.httpGitRequest('/git/server_root', 'GET', null);
            const responseData = yield response.json();
            serverRootResultCache = coreutils_1.PathExt.join(coreutils_1.PathExt.relative(responseData['server_root'], topRepoPath), path);
            return serverRootResultCache;
        }
    });
}
exports.getRelativeFilePath = getRelativeFilePath;
/**
 * Visible for testing.
 */
function clearCache() {
    serverRootResultCache = null;
}
exports.clearCache = clearCache;
